import 'package:embrace/embrace.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;

class NetworkDemo extends StatefulWidget {
  const NetworkDemo({Key? key}) : super(key: key);

  @override
  State<NetworkDemo> createState() => _NetworkDemoState();
}

class _NetworkDemoState extends State<NetworkDemo> {
  final TextEditingController _controller = TextEditingController();

  @override
  void initState() {
    super.initState();
    _controller.text = 'https://httpbin.org/get';
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Network')),
      body: Padding(
        padding: const EdgeInsets.all(8),
        child: Column(
          children: <Widget>[
            TextFormField(
              controller: _controller,
            ),
            ElevatedButton(
              onPressed: () => sendAndLogRequest(HttpMethod.get),
              child: const Text('Get'),
            ),
            ElevatedButton(
              onPressed: () => sendAndLogRequest(HttpMethod.post),
              child: const Text('Post'),
            ),
            ElevatedButton(
              onPressed: () => sendAndLogRequest(HttpMethod.put),
              child: const Text('Put'),
            ),
            ElevatedButton(
              onPressed: () => sendAndLogRequest(HttpMethod.patch),
              child: const Text('Patch'),
            ),
            ElevatedButton(
              onPressed: () => sendAndLogRequest(HttpMethod.delete),
              child: const Text('Delete'),
            ),
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  Future<void> sendAndLogRequest(HttpMethod method) async {
    final startTime = DateTime.now().millisecondsSinceEpoch;
    try {
      final url = Uri.parse(_controller.text);

      http.Response response;
      switch (method) {
        case HttpMethod.put:
          response = await http.put(url);
          break;

        case HttpMethod.post:
          response = await http.post(url);
          break;

        case HttpMethod.patch:
          response = await http.patch(url);
          break;

        case HttpMethod.delete:
          response = await http.delete(url);
          break;

        case HttpMethod.get:
          response = await http.get(url);
          break;
        case HttpMethod.other:
          response = await http.get(url);
          break;
      }

      final endTime = DateTime.now().millisecondsSinceEpoch;
      final isSuccess = (response.statusCode ~/ 100) == 2;
      final errorMessage = isSuccess ? null : response.reasonPhrase;

      Embrace.instance.logNetworkRequest(
        url: url.toString(),
        method: method,
        startTime: startTime,
        endTime: endTime,
        bytesSent: response.request?.contentLength ?? 0,
        bytesReceived: response.contentLength ?? 0,
        statusCode: response.statusCode,
        error: errorMessage,
      );
    } on Exception catch (e) {
      final endTime = DateTime.now().millisecondsSinceEpoch;
      Embrace.instance.logNetworkRequest(
        url: _controller.text,
        method: method,
        startTime: startTime,
        endTime: endTime,
        bytesSent: 0,
        bytesReceived: 0,
        statusCode: 0,
        error: e.toString(),
      );
    }
  }
}
